﻿using System;
using System.Collections;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;
using System.Text.RegularExpressions;

public partial class FunctionsAndProcedures
{
    [SqlFunction]
    public static SqlInt32 REGEXP_INSTR(SqlString data, SqlString model)
    {
        Regex reg = new Regex(model.ToString());

        Match rm = reg.Match(data.ToString());
        if ((rm != null) && (rm.Success))
        {
            return rm.Index + 1;
        }
        return 0;
    }
    private class FieldResult
    {
        public FieldResult(SqlString schema, SqlString table, SqlString column)
        {
            Schema = schema;
            Table = table;
            Column = column;
        }
        public SqlString Schema { get; set; }
        public SqlString Table { get; set; }
        public SqlString Column { get; set; }
    }

    [SqlFunction(
        DataAccess = DataAccessKind.Read,
        SystemDataAccess = SystemDataAccessKind.Read,
        FillRowMethodName = "FindTableFields_FillRow",
        TableDefinition = "Schema_name nvarchar(128), " +
            "Table_name nvarchar(128), " +
            "Column_name nvarchar(128)")]
        
    public static IEnumerable FindTableFields(SqlString model)
    {
        ArrayList resultCollection = new ArrayList();

        using (SqlConnection connection = new SqlConnection("context connection=true"))
        {
            connection.Open();

            using (SqlCommand selectColumns = new SqlCommand(
                "SELECT " +
                "[TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME] " +
                "FROM [INFORMATION_SCHEMA].[COLUMNS] " +
                "WHERE dbo.REGEXP_INSTR([COLUMN_NAME], @model) >0",
                connection))
            {
                selectColumns.Parameters.AddWithValue(
                    "model",
                    model);

                using (SqlDataReader columnReader = selectColumns.ExecuteReader())
                {
                    while (columnReader.Read())
                    {
                        resultCollection.Add(new FieldResult(
                                columnReader.GetSqlString(0),
                                columnReader.GetSqlString(1),
                                columnReader.GetSqlString(2)));
                    }
                }
            }
        }
        return resultCollection;
    }

    public static void FindTableFields_FillRow(object fieldResultObj, out SqlString schema, out SqlString table, out SqlString column)
    {
        FieldResult fieldResult = (FieldResult)fieldResultObj;

        schema = fieldResult.Schema;
        table = fieldResult.Table;
        column = fieldResult.Column;
    }
    // Create table sample_table(id int, data_value nvarchar(128))
    [SqlProcedure()]
    public static void InsertOrUpdateTableProc(SqlInt32 id, SqlString data)
    {
        using (SqlConnection conn = new SqlConnection("context connection=true"))
        {
            SqlCommand InsertCommand = new SqlCommand(
                "insert into sample_table(id, data_value) " +
                "values(@id, @data)", conn);
            InsertCommand.Parameters.AddWithValue("id", id);
            InsertCommand.Parameters.AddWithValue("data", data);
            SqlCommand UpdateCommand = new SqlCommand(
                "update sample_table " +
                "set data_value = @data " +
                "where id = @id", conn);
            UpdateCommand.Parameters.AddWithValue("id", id);
            UpdateCommand.Parameters.AddWithValue("data", data);
            conn.Open();
            if (UpdateCommand.ExecuteNonQuery() == 0)
            {
                InsertCommand.ExecuteNonQuery();
            }
            conn.Close();
        }
    }
}
