﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;

[Serializable]
[SqlUserDefinedType(Format.Native, IsByteOrdered = true)]
public struct SqlComplex : INullable
{
    private bool _null;
    private double _real;
    private double _imaginary;

    public override string ToString()
    {
        if (_null)
        {
            return "NULL";
        }
        return _real.ToString() + (_imaginary < 0 ? "-" : "+") + _imaginary.ToString() + "i";
    }

    public bool IsNull
    {
        get
        {
            return _null;
        }
    }
    public double Real
    {
        get
        {
            return _real;
        }
        set
        {
            _real = value;
        }
    }
    public double Imaginary
    {
        get
        {
            return _imaginary;
        }
        set
        {
            _imaginary = value;
        }
    }
    public static SqlComplex Null
    {
        get
        {
            SqlComplex h = new SqlComplex();
            h._null = true;
            return h;
        }
    }

    public static SqlComplex Parse(SqlString s)
    {
        if (s.IsNull)
            return Null;
        SqlComplex u = new SqlComplex();
        string ss = s.ToString().Replace("i", "").Replace("I", "").Replace(" ", "");
        double sign = 1;
        if (ss.StartsWith("+"))
        {
            ss = ss.Substring(1);
        }
        else if (ss.StartsWith("-"))
        {
            sign = -1;
            ss = ss.Substring(1);
        }
        int p2 = ss.IndexOfAny(new char[] { '+', '-' });
        double v = sign * double.Parse(ss.Substring(0, p2));
        u._real = v;
        ss = ss.Substring(p2);
        sign = 1;
        if (ss.StartsWith("+"))
        {
            ss = ss.Substring(1);
        }
        else if (ss.StartsWith("-"))
        {
            sign = -1;
            ss = ss.Substring(1);
        }
        v = sign * double.Parse(ss);
        u._imaginary = v;
        u._null = false;
        return u;
    }
    [SqlMethod(OnNullCall = false)]
    public double Module()
    {
        return Math.Sqrt(_real * _imaginary);
    }
    [SqlMethod(OnNullCall = false, IsMutator = true)]
    public void SetValue(double r, double i)
    {
        _real = r;
        _imaginary = i;
        _null = false;
    }
    [SqlMethod(OnNullCall = false)]
    public SqlComplex Add(SqlComplex other)
    {
        SqlComplex c = new SqlComplex();
        c.Real = Real + other.Real;
        c.Imaginary = Imaginary + other.Imaginary;
        return c;
    }
}